/*
 *  OpenVPN-GUI -- A Windows GUI for OpenVPN.
 *
 *  Copyright (C) 2004 Mathias Sundman <mathias@nilings.se>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program (see the file COPYING included with this
 *  distribution); if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <windows.h>
#include "tray.h"
#include "openvpn.h"
#include <stdio.h>

/* bool definitions */
#define bool int
#define true 1
#define false 0

/* snprintf with guaranteed null termination */
#define mysnprintf(out, args...) \
        { \
           snprintf (out, sizeof(out), args); \
           out [sizeof (out) - 1] = '\0'; \
        }


/* clear an object */
#define CLEAR(x) memset(&(x), 0, sizeof(x))


int MyStartService()
{
  char msg[200];

  SC_HANDLE schSCManager;
  SC_HANDLE schService;
  SERVICE_STATUS ssStatus; 
  DWORD dwOldCheckPoint; 
  DWORD dwStartTickCount;
  DWORD dwWaitTime;


    // Open a handle to the SC Manager database. 
    schSCManager = OpenSCManager( 
       NULL,                    // local machine 
       NULL,                    // ServicesActive database 
       SC_MANAGER_ALL_ACCESS);  // full access rights 
   
    if (NULL == schSCManager) {
       mysnprintf(msg, "OpenSCManager failed. You need Administrator rights to start a service.");
       MessageBox(NULL, msg, "OpenVPN-GUI", MB_OK);
       return(false);
    }

    schService = OpenService( 
        schSCManager,          // SCM database 
        "OpenVPNService",     // service name
        SERVICE_ALL_ACCESS); 
 
    if (schService == NULL) {
      MessageBox(NULL, "Failed to open \"OpenVPNService\"", "OpenVPN-GUI", MB_OK);
      return(false);
    }
 
    if (!StartService(
            schService,  // handle to service 
            0,           // number of arguments 
            NULL) )      // no arguments 
    {
      MessageBox(NULL, "Failed to start \"OpenVPN Service\"", "OpenVPN-GUI", MB_OK);
      return(false);
    }
    else 
    {
        //printf("Service start pending.\n"); 
    }
 
    // Check the status until the service is no longer start pending. 
    if (!QueryServiceStatus( 
            schService,   // handle to service 
            &ssStatus) )  // address of status information structure
    {
      MessageBox(NULL, "Failed to query service status.", "OpenVPN-GUI", MB_OK);
      return(false);
    }
 
    // Save the tick count and initial checkpoint.
    dwStartTickCount = GetTickCount();
    dwOldCheckPoint = ssStatus.dwCheckPoint;

    while (ssStatus.dwCurrentState == SERVICE_START_PENDING) 
    { 
        // Do not wait longer than the wait hint. A good interval is 
        // one tenth the wait hint, but no less than 1 second and no 
        // more than 5 seconds. 
 
        dwWaitTime = ssStatus.dwWaitHint / 10;

        if( dwWaitTime < 1000 )
            dwWaitTime = 1000;
        else if ( dwWaitTime > 5000 )
            dwWaitTime = 5000;

        Sleep( dwWaitTime );

        // Check the status again. 
        if (!QueryServiceStatus( 
                schService,   // handle to service 
                &ssStatus) )  // address of structure
            break; 
 
        if ( ssStatus.dwCheckPoint > dwOldCheckPoint )
        {
            // The service is making progress.
            dwStartTickCount = GetTickCount();
            dwOldCheckPoint = ssStatus.dwCheckPoint;
        }
        else
        {
            if(GetTickCount()-dwStartTickCount > ssStatus.dwWaitHint)
            {
                // No progress made within the wait hint
                break;
            }
        }
    } 

    CloseServiceHandle(schService); 
    CloseServiceHandle(schSCManager);

    if (ssStatus.dwCurrentState != SERVICE_RUNNING) 
    { 
        MessageBox(NULL, "OpenVPN Service failed to start.", "OpenVPN-GUI", MB_OK); 
        return(false);
    } 

    SetServiceMenu(true);
    return(true);
}

int MyStopService()
{
  char msg[200];

  SC_HANDLE schSCManager;
  SC_HANDLE schService;
  SERVICE_STATUS ssStatus; 
  DWORD dwOldCheckPoint; 
  DWORD dwStartTickCount;
  DWORD dwWaitTime;


    // Open a handle to the SC Manager database. 
    schSCManager = OpenSCManager( 
       NULL,                    // local machine 
       NULL,                    // ServicesActive database 
       SC_MANAGER_ALL_ACCESS);  // full access rights 
   
    if (NULL == schSCManager) {
       mysnprintf(msg, "OpenSCManager failed (%d)", (int) GetLastError());
       MessageBox(NULL, msg, "OpenVPN GUI", MB_OK);
       return(false);
    }

    schService = OpenService( 
        schSCManager,          // SCM database 
        "OpenVPNService",     // service name
        SERVICE_STOP); 
 
    if (schService == NULL) {
      MessageBox(NULL, "Failed to open OpenVPNService", "OpenVPN-GUI", MB_OK);
      return(false);
    }

    if (!ControlService( 
            schService,   // handle to service 
            SERVICE_CONTROL_STOP,   // control value to send 
            &ssStatus) )  // address of status info 
    {
      MessageBox(NULL, "Failed to stop OpenVPN Service", "OpenVPN-GUI", MB_OK);
      return(false);
    }

    SetServiceMenu(false); 
    return(true);
}

int MyReStartService()
{
    MyStopService();
    Sleep(1000);
    if (MyStartService()) {
       MessageBox(NULL, "OpenVPN Service Restarted.", "OpenVPN-GUI", MB_OK);
       return(true);
    }

    return(false);
}


int CheckServiceStatus()
{
  char msg[200];

  SC_HANDLE schSCManager;
  SC_HANDLE schService;
  SERVICE_STATUS ssStatus; 
  DWORD dwOldCheckPoint; 
  DWORD dwStartTickCount;
  DWORD dwWaitTime;


    // Open a handle to the SC Manager database. 
    schSCManager = OpenSCManager( 
      NULL,                    // local machine 
      NULL,                    // ServicesActive database 
      SC_MANAGER_ALL_ACCESS);  // full access rights 
   
    if (NULL == schSCManager) {
      //mysnprintf(msg, "OpenSCManager failed (%d)", (int) GetLastError());
      //MessageBox(NULL, msg, "OpenVPN GUI", MB_OK);
      SetServiceMenuDisabled();
      return(false);
    }

    schService = OpenService( 
      schSCManager,          // SCM database 
      "OpenVPNService",     // service name
      SERVICE_QUERY_STATUS); 
 
    if (schService == NULL) {
      MessageBox(NULL, "Failed to open OpenVPNService", "OpenVPN-GUI", MB_OK);
      SetServiceMenuDisabled();
      return(false);
    }

    if (!QueryServiceStatus( 
            schService,   // handle to service 
            &ssStatus) )  // address of status information structure
    {
      MessageBox(NULL, "Failed to query service status.", "OpenVPN-GUI", MB_OK);
      return(false);
    }
 
    if (ssStatus.dwCurrentState == SERVICE_RUNNING) 
    {
        SetServiceMenu(true);
        return(true);
    }
    else 
    { 
        SetServiceMenu(false);
        return(false);
    } 
} 
